--
-- Create user
--
-- CREATE USER "ethereal" ENCRYPTED PASSWORD '0bddS5gaMdpNI' NOCREATEDB NOCREATEUSER;

DROP SCHEMA ethereal CASCADE;

--
-- Create Schema to carry all objects
--
CREATE SCHEMA ethereal AUTHORIZATION ethereal;

-- SEQUENCES
--
-- Single pass
CREATE SEQUENCE ethereal.seqComment MAXVALUE 2147483647;
CREATE SEQUENCE ethereal.seqFortune MAXVALUE 2147483647;
CREATE SEQUENCE ethereal.seqGallery MAXVALUE 2147483647;
CREATE SEQUENCE ethereal.seqMail    MAXVALUE 2147483647;
CREATE SEQUENCE ethereal.seqOption  MAXVALUE 2147483647;
CREATE SEQUENCE ethereal.seqPoll    MAXVALUE 2147483647;
CREATE SEQUENCE ethereal.seqWork    MAXVALUE 2147483647 START 15;


--
-- Loop
CREATE SEQUENCE ethereal.seqAudit   MAXVALUE 2147483647 CYCLE;
CREATE SEQUENCE ethereal.seqBoard   MAXVALUE 2147483647 CYCLE;
CREATE SEQUENCE ethereal.seqPost    MAXVALUE 2147483647 CYCLE;
CREATE SEQUENCE ethereal.seqWeblog  MAXVALUE 2147483647 CYCLE;


--
--
-- NAME        : Contact
-- REFERENCES  : NIL
--
-- DESCRIPTION : Contact is used to allow puppeteers to broadcast information to
--               other patrons.   The PuppeteerContact table is in fact dependent
--               on this.

CREATE TABLE ethereal.Contact (
  ContactName          VARCHAR(25)   NOT NULL,
  ContactType          VARCHAR(10),
  ContactFilter        VARCHAR(10)   NOT NULL DEFAULT 'none',
  CONSTRAINT pkContact               PRIMARY KEY (ContactName),
  CONSTRAINT enumContactFilter       CHECK (ContactFilter IN ('email','http','none','numeric'))
) WITHOUT OIDS;


--
--
-- NAME        : Font
-- REFERENCES  : NIL
--
-- DESCRIPTION : An easy way to manage font face, size and colour.

CREATE TABLE ethereal.Font (
  FontName             VARCHAR(25)   NOT NULL,
  FontType             VARCHAR(5)    NOT NULL,
  FontCode             VARCHAR(35),
  CONSTRAINT pkFont                  PRIMARY KEY(FontName, FontType),
  CONSTRAINT enumFontType            CHECK (FontType in ('color','face','size'))
) WITHOUT OIDS;


--
--
-- NAME        : Fortune
-- REFERENCES  : NIL
--
-- DESCRIPTION : Completely independant table that carries with it no associations
--               which would prevent addition.

CREATE TABLE ethereal.Fortune (
  FortuneNum           INT           NOT NULL DEFAULT NEXTVAL('ethereal.seqFortune'),
  FortuneType          VARCHAR(15)   NOT NULL,
  FortuneMsg           TEXT,
  CONSTRAINT pkFortune               PRIMARY KEY (FortuneNum)
) WITHOUT OIDS;


-- STANDARD INDEX
CREATE INDEX idxFortune ON ethereal.Fortune
(
  FortuneType
);


--
--
-- NAME        : Genre
-- REFERENCES  : NIL
--
-- DESCRIPTION : Mostly used for the organization and grouping of realms and
--               galleries.  Some tables will be dependant on this, at least
--               partially such as Gallery and Realm.

CREATE TABLE ethereal.Genre (
  GenreName            VARCHAR(25)  NOT NULL,
  GenreOrder           VARCHAR(5)   NOT NULL DEFAULT 'realm',
  GenreDescription     VARCHAR(250),
  CONSTRAINT pkGenre                PRIMARY KEY (GenreName, GenreOrder),
  CONSTRAINT enumGenreOrder         CHECK (GenreOrder IN ('realm','image','text'))
) WITHOUT OIDS;



--
--
-- NAME        : Tag
-- REFERENCES  : NIL
--
-- DESCRIPTION : Tag is a completely independent table which is used to store key/pair
--               values for the customized look and feel of the site itself.

CREATE TABLE ethereal.Tag (
  TagName              VARCHAR(25)    NOT NULL,
  TagType              VARCHAR(15)    NOT NULL DEFAULT 'system',
  TagData              TEXT,
  CONSTRAINT pkTag                    PRIMARY KEY (TagName, TagType),
  CONSTRAINT enumTagType              CHECK (TagType IN ('admin','board','configuration','gallery','home','login','mail','poll','portal','realm','system','template','vcodes'))
) WITHOUT OIDS;

--
--
-- NAME        : Rotate
-- REFERENCES  : NIL
--
-- DESCRIPTION : Rotate is an independant table that exists to generate various 
--               material that will be rotated as to provide an internal method for
--               varying displayed information.

CREATE TABLE ethereal.Rotate (
  RotateName           VARCHAR(25)    NOT NULL,
  RotateZone           VARCHAR(8)     NOT NULL DEFAULT 'banner',
  RotateConduct        VARCHAR(8)     NOT NULL DEFAULT 'static',
  RotateTimestamp      TIMESTAMP      NOT NULL DEFAULT LOCALTIMESTAMP,
  RotateData           TEXT,
  CONSTRAINT pkRotate                 PRIMARY KEY (RotateName, RotateZone),
  CONSTRAINT enumRotateZone           CHECK (RotateZone IN ('banner','gallery','portal','ssi')),
  CONSTRAINT enumRotateConduct        CHECK (RotateConduct IN ('image','static','text'))
) WITHOUT OIDS;


-- ================================================================================== --
-- POLL                                                                               --
-- ================================================================================== --

--
--
-- NAME        : Poll
-- REFERENCES  : NIL
--
-- DESCRIPTION : Poll is the encompassing entity which links Options and eventually
--               votes are they are done.

CREATE TABLE ethereal.Poll (
  PollID               INT           NOT NULL DEFAULT NEXTVAL('ethereal.seqPoll'),
  PollName             VARCHAR(20)   NOT NULL,
  PollQuestion         VARCHAR(255)  NOT NULL,
  CONSTRAINT pkPoll                  PRIMARY KEY (PollID)
) WITHOUT OIDS;


--
--
-- NAME        : Poll Option
-- REFERENCES  : Poll
--
-- DESCRIPTION : Poll Option lists all the choices for the polls.

CREATE TABLE ethereal.PollOption (
  PollID               INT           NOT NULL,
  OptionID             INT           NOT NULL DEFAULT NEXTVAL('ethereal.seqOption'),
  OptionName           VARCHAR(50)   NOT NULL,
  CONSTRAINT pkPollOption            PRIMARY KEY (PollID, OptionID),
  CONSTRAINT fkPollOption            FOREIGN KEY (PollID)
    REFERENCES ethereal.Poll (PollID)
      ON DELETE CASCADE
      ON UPDATE CASCADE
) WITHOUT OIDS;


--
--
-- NAME        : Poll Vote
-- REFERENCES  : Poll
--               Poll Option
--
-- DESCRIPTION : Poll vote stores the physical votes made on the Polls.

CREATE TABLE ethereal.PollVote (
  PollID               INT           NOT NULL,
  VoteHostname         INET          NOT NULL,
  OptionID             INT           NOT NULL,
  CONSTRAINT pkPollVote              PRIMARY KEY (PollID, VoteHostname),
  CONSTRAINT fkPollVote_PollID       FOREIGN KEY (PollID, OptionID)
    REFERENCES ethereal.PollOption (PollID, OptionID)
      ON DELETE CASCADE
      ON UPDATE CASCADE
) WITHOUT OIDS;



-- ================================================================================== --
-- PUPPETEER                                                                          --
-- ================================================================================== --

--
--
-- NAME        : Puppeteer
-- REFERENCES  : NIL
--
-- DESCRIPTION : Puppeteer is the primary holder of all information of the chat site
--               meaning that all activity is done through a puppeteer.


CREATE TABLE ethereal.Puppeteer
(
  PuppeteerLogin       VARCHAR(10)   NOT NULL,
  PuppeteerPassword    VARCHAR(13)   NOT NULL,
  PuppeteerName        VARCHAR(30)   NOT NULL,
  PuppeteerAlias       VARCHAR(10),
  PuppeteerEMail       VARCHAR(45)   NOT NULL,
  PuppeteerTimestamp   TIMESTAMP,
  PuppeteerHostname    INET,
  PuppeteerBan         VARCHAR(3)    NOT NULL DEFAULT 'no',
  PuppeteerVerified    VARCHAR(3)    NOT NULL DEFAULT 'no',
  PuppeteerChatPref    VARCHAR(10)   NOT NULL DEFAULT 'stream',
  PuppeteerFormatting  VARCHAR(9)    NOT NULL DEFAULT 'complete',
  PuppeteerJavascript  VARCHAR(3)    NOT NULL DEFAULT 'yes',
  PuppeteerPrivacy     VARCHAR(6)    NOT NULL DEFAULT 'no',
  PuppeteerAdminPrivs  VARCHAR(3)    NOT NULL DEFAULT 'no',
  PuppeteerSuperPrivs  VARCHAR(3)    NOT NULL DEFAULT 'no',
  PuppeteerModPrivs    VARCHAR(3)    NOT NULL DEFAULT 'no',
  PuppeteerIgnore      VARCHAR(3)    NOT NULL DEFAULT 'yes',
  PuppeteerHome        VARCHAR(200),
  PuppeteerHomeSpan    TIMESTAMP     NOT NULL DEFAULT LOCALTIMESTAMP,
  PuppeteerRealm       VARCHAR(30),
  CONSTRAINT pkPuppeteer             PRIMARY KEY(PuppeteerLogin),
  CONSTRAINT enumPuppeteerBan        CHECK (PuppeteerBan        IN ('yes','no')),
  CONSTRAINT enumPuppeteerVerified   CHECK (PuppeteerVerified   IN ('yes','no')),
  CONSTRAINT enumPuppeteerChatPref   CHECK (PuppeteerChatPref   IN ('basic','framed','stream','autoscroll')),
  CONSTRAINT enumPuppeteerFormatting CHECK (PuppeteerFormatting IN ('complete','imageless','spartan')),
  CONSTRAINT enumPuppeteerJavascript CHECK (PuppeteerJavascript IN ('yes','no')),
  CONSTRAINT enumPuppeteerPrivacy    CHECK (PuppeteerPrivacy    IN ('yes','no','global')),
  CONSTRAINT enumPuppeteerAdminPrivs CHECK (PuppeteerAdminPrivs IN ('yes','no')),
  CONSTRAINT enumPuppeteerSuperPrivs CHECK (PuppeteerSuperPrivs IN ('yes','no')),
  CONSTRAINT enumPuppeteerModPrivs   CHECK (PuppeteerModPrivs   IN ('yes','no')),
  CONSTRAINT enumPuppeteerIgnore     CHECK (PuppeteerIgnore     IN ('yes','no'))
) WITHOUT OIDS;

-- FUNCTION
CREATE UNIQUE INDEX fncPuppeteer_Login ON ethereal.Puppeteer
(
  lower(PuppeteerLogin)
);

CREATE INDEX fncPuppeteer_Name ON ethereal.Puppeteer
(
  lower(PuppeteerName)
);

CREATE INDEX fncPuppeteer_Home ON ethereal.Puppeteer
(
  lower(PuppeteerHome)
);

CREATE INDEX fncPuppeteer_Email ON ethereal.Puppeteer
(
  lower(PuppeteerEmail)
);


-- STANDARD INDEX
CREATE INDEX idxPuppeteer_Ident_Name ON ethereal.Puppeteer
(
  PuppeteerName
);

CREATE INDEX idxPuppeteer_Ident_Home ON ethereal.Puppeteer
(
  PuppeteerHome
);

CREATE INDEX idxPuppeteer_Ident_Email ON ethereal.Puppeteer
(
  PuppeteerEMail
);

CREATE INDEX idxPuppeteer_Time ON ethereal.Puppeteer
(
  PuppeteerTimestamp
);

CREATE INDEX idxPuppeteer_Privs_Admin ON ethereal.Puppeteer
(
  PuppeteerAdminPrivs
);

CREATE INDEX idxPuppeteer_Privs_Mod ON ethereal.Puppeteer
(
  PuppeteerModPrivs
);

CREATE INDEX idxPuppeteer_Privs_Super ON ethereal.Puppeteer
(
  PuppeteerSuperPrivs
);

CREATE INDEX idxPuppeteer_Privs_Verified ON ethereal.Puppeteer
(
  PuppeteerVerified
);


--
--
-- NAME        : Param
-- REFERENCES  : Puppeteer
--
-- DESCRIPTION : Transaction is a way for Ethereal Realms to secure and lock down the
--               system while reducing overall transmitted traffic.


CREATE TABLE ethereal.Param (
  ParamNumber          INTEGER       NOT NULL,
  PuppeteerLogin       VARCHAR(10)   NOT NULL,
  ParamValue           TEXT          NOT NULL,
  ParamTimestamp       TIMESTAMP     NOT NULL DEFAULT LOCALTIMESTAMP,
  CONSTRAINT PkParam                 PRIMARY KEY (ParamNumber, PuppeteerLogin),
  CONSTRAINT FkParam                 FOREIGN KEY (PuppeteerLogin)
    REFERENCES ethereal.Puppeteer (PuppeteerLogin)
      ON DELETE CASCADE
      ON UPDATE CASCADE
) WITHOUT OIDS;

-- Standard Index
CREATE INDEX idxParam_Timestamp ON ethereal.Param
(
  ParamTimestamp
);


--
--
-- NAME        : Puppeteer Contact
-- REFERENCES  : Puppeteer
--
-- DESCRIPTION : Contact allows a puppeteer to store specific contact information
--               to be made public.

CREATE TABLE ethereal.PuppeteerContact (
  ContactName          VARCHAR(25)   NOT NULL,
  PuppeteerLogin       VARCHAR(10)   NOT NULL,
  ContactData          VARCHAR(200),
  CONSTRAINT pkPuppeteerContact      PRIMARY KEY (ContactName, PuppeteerLogin),
  CONSTRAINT fkPuppetContact_Name    FOREIGN KEY (ContactName)
    REFERENCES ethereal.Contact (ContactName)
      ON DELETE CASCADE
      ON UPDATE CASCADE,
  CONSTRAINT fkPuppetContact_Login   FOREIGN KEY (PuppeteerLogin)
    REFERENCES ethereal.Puppeteer (PuppeteerLogin)
      ON DELETE CASCADE
      ON UPDATE CASCADE
) WITHOUT OIDS;


--
--
-- NAME        : Puppeteer Registration
-- REFERENCES  : Puppeteer
--
-- DESCRIPTION : This is a non normalized table, meaning that it should really be a
--               part of Puppeteer.  But is seperate as it is seldomly used, and then
--               only for new users.

CREATE TABLE ethereal.PuppeteerRegistration (
  PuppeteerLogin       VARCHAR(10)   NOT NULL,
  PuppeteerConfCode    VARCHAR(10)   NOT NULL,
  PuppeteerTimeStamp   TIMESTAMP     NOT NULL DEFAULT LOCALTIMESTAMP,
  CONSTRAINT pkPuppeteerRegistration PRIMARY KEY (PuppeteerLogin),
  CONSTRAINT fkPuppeteerRegistration FOREIGN KEY (PuppeteerLogin)
    REFERENCES ethereal.Puppeteer (PuppeteerLogin)
      ON DELETE CASCADE
      ON UPDATE CASCADE
) WITHOUT OIDS;


--
--
-- NAME        : Association
-- REFERENCES  : Puppeteer
--
-- DESCRIPTION : Groups are classifications for puppets as specified by the user
--               himself.  This essentially allows one to organize puppets in
--               a non-standard way.

CREATE TABLE ethereal.Association (
  PuppeteerLogin       VARCHAR(10)   NOT NULL,
  AssociationName      VARCHAR(10)   NOT NULL,
  CONSTRAINT pkAssociation PRIMARY KEY (PuppeteerLogin, AssociationName),
  CONSTRAINT fkAssociation FOREIGN KEY (PuppeteerLogin)
    REFERENCES ethereal.Puppeteer (PuppeteerLogin)
      ON DELETE CASCADE
      ON UPDATE CASCADE
) WITHOUT OIDS;


--
--
-- NAME        : Audit
-- REFERENCES  : Puppeteer
--
-- DESCRIPTION : Auditing is a vital part of Ethereal Realms, allowing actions to be
--               tracked.

CREATE TABLE ethereal.Audit (
  AuditNumber          INT           NOT NULL DEFAULT NEXTVAL('ethereal.seqAudit'),
  PuppeteerLogin       VARCHAR(10)   NOT NULL,
  PuppeteerHostname    INET,
  AuditAction          VARCHAR(6)    NOT NULL DEFAULT 'auth',
  AuditCode            SMALLINT      NOT NULL DEFAULT '1000',
  AuditAdditional      VARCHAR(50),
  AuditRealm           VARCHAR(30)   NOT NULL DEFAULT 'system',
  AuditTimestamp       TIMESTAMP     NOT NULL DEFAULT LOCALTIMESTAMP,
  CONSTRAINT pkAudit                 PRIMARY KEY (AuditNumber),
  CONSTRAINT enumAction CHECK (AuditAction IN ('action','auth','delete','insert','update')),
  CONSTRAINT numCode    CHECK (AuditCode >= 1000 AND AuditCode < 10000 )
) WITHOUT OIDS;

-- Standard Index
CREATE INDEX idxAudit_Action ON ethereal.Audit
(
  AuditAction
);

CREATE INDEX idxAudit_Hostname ON ethereal.Audit
(
  PuppeteerHostname
);

-- Standard Index
CREATE INDEX idxAudit_Realm ON ethereal.Audit
(
  AuditRealm
);

CREATE INDEX idxAudit_Timestamp ON ethereal.Audit
(
  AuditTimestamp
);

-- MULTI COLUMN
CREATE INDEX idxAudit_CodeTimestamp ON ethereal.Audit
(
  AuditCode,
  AuditTimestamp
);


--
--
-- NAME        : Transaction
-- REFERENCES  : Puppeteer
--
-- DESCRIPTION : Transaction allows the system to store donations made to the system.

CREATE TABLE ethereal.Transaction (
  TransIdentifier      VARCHAR(20)   NOT NULL,
  PuppeteerLogin       VARCHAR(10)   NOT NULL,
  TransGross           NUMERIC(9,2)  NOT NULL,
  TransCur             CHAR(3)       NOT NULL,
  TransTimestamp       TIMESTAMP     NOT NULL DEFAULT LOCALTIMESTAMP,
  CONSTRAINT pkTrans                 PRIMARY KEY (TransIdentifier),
  CONSTRAINT fkTrans                 FOREIGN KEY (PuppeteerLogin)
    REFERENCES ethereal.Puppeteer (PuppeteerLogin)
      ON DELETE CASCADE
      ON UPDATE CASCADE
) WITHOUT OIDS;


-- ================================================================================== --
-- GALLERY                                                                            --
-- ================================================================================== --

--
--
-- NAME        : Gallery
-- REFERENCES  : Puppeteer
--
-- DESCRIPTION : Galleries are a way for Puppeteers to present works to the general
--               population.

CREATE TABLE ethereal.Gallery (
  GalleryID            INT           NOT NULL DEFAULT NEXTVAL('ethereal.seqGallery'),
  PuppeteerLogin       VARCHAR(10)   NOT NULL,
  GalleryPenName       VARCHAR(30)   NOT NULL,
  GalleryName          VARCHAR(30)   NOT NULL,
  GalleryRating        VARCHAR(5)    NOT NULL,
  GalleryDescription   VARCHAR(250),
  GalleryComments      VARCHAR(7)    NOT NULL DEFAULT 'combo',
  GallerySort          VARCHAR(7)    NOT NULL DEFAULT 'alpha',
  GalleryType          VARCHAR(7)    NOT NULL DEFAULT 'initial',
  GalleryDisplay       SMALLINT,
  GalleryCreated       TIMESTAMP     NOT NULL DEFAULT LOCALTIMESTAMP,
  GalleryUpdated       TIMESTAMP     NOT NULL DEFAULT LOCALTIMESTAMP,
  CONSTRAINT pkGallery               PRIMARY KEY (GalleryID),
  CONSTRAINT fkGallery_Login         FOREIGN KEY (PuppeteerLogin)
    REFERENCES ethereal.Puppeteer (PuppeteerLogin)
      ON DELETE CASCADE
      ON UPDATE CASCADE,
  CONSTRAINT enumGalleryComments     CHECK (GalleryComments IN ('combo','comment','none','rating')),
  CONSTRAINT enumGallerySort         CHECK (GallerySort IN ('alpha','date','manual','rating')),
  CONSTRAINT enumGalleryType         CHECK (GalleryType IN ('image','initial','text'))
) WITHOUT OIDS;

-- FUNCTION INDEX
CREATE INDEX fncGallery_Pen ON ethereal.Gallery
(
  lower(GalleryPenName)
);

CREATE INDEX fncGallery_Name ON ethereal.Gallery
(
  lower(GalleryName)
);

CREATE INDEX fncGallery_Rating ON ethereal.Gallery
(
  lower(GalleryRating)
);


-- STANDARD INDEX
CREATE INDEX idxGallery_Name ON ethereal.Gallery
(
  GalleryName
);

CREATE INDEX idxGallery_Pen ON ethereal.Gallery
(
  GalleryPenName
);

CREATE INDEX idxGallery_Rating ON ethereal.Gallery
(
  GalleryRating
);


--
--
-- NAME        : Work
-- REFERENCES  : Gallery
--
-- DESCRIPTION : Works are located under galleries, and are contained within
--               said galleries in order to present them as one would find in
--               in a museum.

CREATE TABLE ethereal.Work (
  GalleryID            INT           NOT NULL,
  WorkID               INT           NOT NULL DEFAULT NEXTVAL('ethereal.seqWork'),
  WorkName             VARCHAR(30)   NOT NULL,
  WorkDescription      VARCHAR(250),
  WorkSort             SMALLINT,
  WorkImage            VARCHAR(200),
  WorkImageThumbnail   VARCHAR(200),
  WorkImageHeight      SMALLINT,
  WorkImageWidth       SMALLINT,
  WorkImageSize        SMALLINT,
  WorkText             TEXT,
  WorkTextPreserve     VARCHAR(3)    NOT NULL DEFAULT 'no',
  WorkTimestamp        TIMESTAMP     NOT NULL DEFAULT LOCALTIMESTAMP,
  WorkRating           SMALLINT      NOT NULL DEFAULT '0',
  CONSTRAINT pkWork                  PRIMARY KEY (GalleryID, WorkID),
  CONSTRAINT fkWork                  FOREIGN KEY (GalleryID)
    REFERENCES ethereal.Gallery (GalleryID)
      ON DELETE CASCADE
      ON UPDATE CASCADE,
  CONSTRAINT enumWorkTextPreserve    CHECK (WorkTextPreserve IN ('yes','no'))
) WITHOUT OIDS;

-- FUNCTION INDEX
CREATE INDEX fncWork_Name ON ethereal.Work
(
  lower(WorkName)
);

CREATE INDEX fncWork_Descr ON ethereal.Work
(
  lower(WorkDescription)
);

-- STANDARD INDEX
CREATE INDEX idxWork_Description ON ethereal.Work
(
  WorkDescription
);

CREATE INDEX idxWork_Name ON ethereal.Work
(
  WorkName
);


--
--
-- NAME        : Comment
-- REFERENCES  : Gallery
--               Work
--
-- DESCRIPTION : Comment allows users to make comments on works located under
--               galleries.

CREATE TABLE ethereal.Comment (
  GalleryID            INT           NOT NULL,
  WorkID               INT           NOT NULL,
  CommentID            INT           NOT NULL DEFAULT NEXTVAL('ethereal.seqComment'),
  PuppetName           VARCHAR(30)   NOT NULL,
  PuppeteerLogin       VARCHAR(10)   NOT NULL,
  CommentTimestamp     TIMESTAMP     NOT NULL DEFAULT LOCALTIMESTAMP,
  CommentPost          TEXT,
  CommentRating        SMALLINT      NOT NULL,
  CONSTRAINT pkComment               PRIMARY KEY (GalleryID, WorkID, CommentID),
  CONSTRAINT fkComment               FOREIGN KEY (GalleryID, WorkID)
    REFERENCES ethereal.Work (GalleryID, WorkID)
      ON DELETE CASCADE
      ON UPDATE CASCADE
) WITHOUT OIDS;


--
--
-- NAME        : Gallery Genre
-- REFERENCES  : Gallery
--               Genre
--
-- DESCRIPTION : This is needed in order to allow for more then one genre for
--               a gallery.  

CREATE TABLE ethereal.GalleryGenre (
  GalleryID            INT          NOT NULL,
  GenreName            VARCHAR(25)  NOT NULL,
  GenreOrder           VARCHAR(5)   NOT NULL,
  CONSTRAINT fkGenre_Gallery        FOREIGN KEY (GalleryID)
    REFERENCES ethereal.Gallery (GalleryID)
      ON DELETE CASCADE
      ON UPDATE CASCADE,
  CONSTRAINT fkGenre_Genre          FOREIGN KEY (GenreName, GenreOrder)
    REFERENCES ethereal.Genre (GenreName, GenreOrder)
      ON DELETE CASCADE
      ON UPDATE CASCADE
) WITHOUT OIDS;

--
--
-- NAME        : Gallery Notify
-- REFERENCES  : Gallery
--               Puppeteer
--
-- DESCRIPTION : This table is used to notify a patron that there has in fact
--               been changes done to the gallery.    Which in turn allows them
--               to keep track of new works without being notified or having to
--               check up on the gallery all the time.

CREATE TABLE ethereal.GalleryNotify (
  GalleryID            INT           NOT NULL,
  PuppeteerLogin       VARCHAR(10)   NOT NULL,
  CONSTRAINT fkNotify_Gallery        FOREIGN KEY (GalleryID)
    REFERENCES ethereal.Gallery (GalleryID)
      ON DELETE CASCADE
      ON UPDATE CASCADE,
  CONSTRAINT fkComment_Puppeteer     FOREIGN KEY (PuppeteerLogin)
    REFERENCES ethereal.Puppeteer (PuppeteerLogin)
      ON DELETE CASCADE
      ON UPDATE CASCADE
) WITHOUT OIDS;


-- ================================================================================== --
-- REALM                                                                              --
-- ================================================================================== --

--
--
-- NAME        : Realm
-- REFERENCES  : NIL
--
-- DESCRIPTION : Primary identifying component for all of the supporting
--               realms.  Allowing puppets to play within said containers.

CREATE TABLE ethereal.Realm (
  RealmName            VARCHAR(30)   NOT NULL,
  RealmAlias           VARCHAR(30),
  RealmDescription     VARCHAR(250),
  RealmHomepage        VARCHAR(255),
  RealmRating          VARCHAR(5)    NOT NULL DEFAULT 'PG',
  RealmGenre           VARCHAR(25)   NOT NULL DEFAULT 'General',
  RealmPassword        VARCHAR(13),
  RealmArchive         VARCHAR(45),
  RealmAccess          VARCHAR(10)   NOT NULL DEFAULT 'relaxed',
  RealmVerified        VARCHAR(3)    NOT NULL DEFAULT 'no',
  RealmBanner          VARCHAR(7)    NOT NULL DEFAULT 'display',
  RealmDateLoc         VARCHAR(7)    NOT NULL DEFAULT 'bottom',
  RealmDateFormat      VARCHAR(30)   NOT NULL DEFAULT 'DD-Mon-YYYY, HH24:MI:SS TZ',
  RealmPublic          VARCHAR(6)    NOT NULL DEFAULT 'no',
  RealmLocale          VARCHAR(8)    NOT NULL DEFAULT 'none',
  RealmTags            VARCHAR(10)   NOT NULL DEFAULT 'relaxed',
  RealmLanguage        VARCHAR(10)   NOT NULL DEFAULT 'relaxed',
  RealmFilter          VARCHAR(10)   NOT NULL DEFAULT '****',
  RealmFontColor       TEXT,
  RealmFontFace        TEXT ,
  RealmFontSize        TEXT,
  RealmTextColour      VARCHAR(25)   NOT NULL DEFAULT 'White',
  RealmImageHeight     SMALLINT      NOT NULL DEFAULT '0',
  RealmImageWidth      SMALLINT      NOT NULL DEFAULT '0',
  RealmImageSize       SMALLINT      NOT NULL DEFAULT '0',
  RealmFramesetRows    SMALLINT      NOT NULL DEFAULT '115',
  RealmFramesetCols    SMALLINT      NOT NULL DEFAULT '65',
  RealmBoard           VARCHAR(45),
  RealmAltBoard        VARCHAR(255),
  RealmICAO            VARCHAR(4),
  RealmMETAR           VARCHAR(250),
  RealmTimestamp       TIMESTAMP     NOT NULL DEFAULT LOCALTIMESTAMP,
  CONSTRAINT pkRealm                 PRIMARY KEY (RealmName),
  CONSTRAINT enumRealmAccess         CHECK (RealmAccess   IN ('relaxed','restricted','list')),
  CONSTRAINT enumRealmVerified       CHECK (RealmVerified IN ('yes','no')),
  CONSTRAINT enumRealmBanner         CHECK (RealmBanner   IN ('display','exempt')),
  CONSTRAINT enumRealmDateLoc        CHECK (RealmDateLoc  IN ('bottom','top','removed')),
  CONSTRAINT enumRealmLocale         CHECK (RealmLocale   IN ('none','strict','variable')),
  CONSTRAINT enumRealmPublic         CHECK (RealmPublic   IN ('yes','no','hybrid','alias')),
  CONSTRAINT enumRealmTags           CHECK (RealmTags     IN ('relaxed','restricted')),
  CONSTRAINT enumRealmLanguage       CHECK (RealmLanguage IN ('relaxed','restricted'))
) WITHOUT OIDS;

-- FUNCTION INDEX
CREATE INDEX fncRealm ON ethereal.Realm
(
  lower(RealmName)
);

-- STANDARD INDEX
CREATE INDEX idxRealm_Alias ON ethereal.Realm
(
  RealmAlias
);

CREATE INDEX idxRealm_Public ON ethereal.Realm
(
  RealmPublic
);

CREATE INDEX idxRealm_Timestamp ON ethereal.Realm
(
  RealmTimestamp
);

CREATE INDEX idxRealm_Verified ON ethereal.Realm
(
  RealmVerified
);


-- MULTI COLUMN
CREATE INDEX idxRealm_NamePublicVerified ON ethereal.Realm
(
  RealmName, 
  RealmPublic, 
  RealmVerified
);

--
--
-- NAME        : Board Topic
-- REFERENCES  : Realm
--
-- DESCRIPTION : Board topic allows a realm owner to add topics to their locally
--               hosted board.

CREATE TABLE ethereal.BoardTopic (
  RealmName            VARCHAR(30)   NOT NULL,
  BoardTopic           VARCHAR(25)   NOT NULL,
  BoardDescription     TEXT,
  CONSTRAINT pkBoardTopic            PRIMARY KEY (RealmName, BoardTopic),
  CONSTRAINT fkBoardTopic            FOREIGN KEY (RealmName)
    REFERENCES ethereal.Realm (RealmName)
      ON DELETE CASCADE
      ON UPDATE CASCADE
) WITHOUT OIDS;


--
--
-- NAME        : Gummy
-- REFERENCES  : Realm
--
-- DESCRIPTION : Gummies are used in conjuction with handles during their plays
--               in realm.    This connection to realms is not yet directly
--               applied.

CREATE TABLE ethereal.Gummy (
  GummyName            VARCHAR(20)   NOT NULL,
  GummyClass           VARCHAR(12)   NOT NULL DEFAULT 'general',
  RealmName            VARCHAR(30)   NOT NULL DEFAULT 'system',
  GummyPath            VARCHAR(200)  NOT NULL,
  GummyIdent           VARCHAR(25),
  CONSTRAINT pkGummy                 PRIMARY KEY (GummyName),
  CONSTRAINT fkGummy                 FOREIGN KEY (RealmName)
    REFERENCES ethereal.Realm (RealmName)
      ON DELETE CASCADE
      ON UPDATE CASCADE,
  CONSTRAINT enumGummyClass          CHECK (GummyClass IN ('admin','default','canon','contributor','general','mod','super'))
) WITHOUT OIDS;

-- STANDARD INDEX
CREATE INDEX idxGummy_Class ON ethereal.Gummy
(
       GummyClass
);

CREATE INDEX idxGummy_Ident ON ethereal.Gummy
(
       GummyIdent
);

CREATE INDEX idxGummy_Name ON ethereal.Gummy
(
       GummyName
);

CREATE INDEX idxGummy_Path ON ethereal.Gummy
(
       GummyPath
);


--
--
-- NAME        : Realm Design
-- REFERENCES  : Realm
--
-- DESCRIPTION : Realm design stores realm specific customizations of templates
--               being critical to the design of pages.

CREATE TABLE ethereal.RealmDesign (
  RealmName            VARCHAR(30)   NOT NULL,
  TagName              VARCHAR(20)   NOT NULL,
  DesignContent        TEXT,
  CONSTRAINT pkRealmDesign           PRIMARY KEY (RealmName, TagName),
  CONSTRAINT fkRealmDesign           FOREIGN KEY (RealmName)
    REFERENCES ethereal.Realm (RealmName)
      ON DELETE CASCADE
      ON UPDATE CASCADE
) WITHOUT OIDS;


--
--
-- NAME        : Realm Locale
-- REFERENCES  : Realm
--
-- DESCRIPTION : Realm locale contains a list of locales allowed by specific
--               realms when using restricted mode.

CREATE TABLE ethereal.RealmLocale (
  RealmName            VARCHAR(30)   NOT NULL,
  RealmLocale          VARCHAR(50)   NOT NULL,
  CONSTRAINT pkRealmLocale           PRIMARY KEY (RealmName, RealmLocale),
  CONSTRAINT fkRealmLocale           FOREIGN KEY (RealmName)
    REFERENCES ethereal.Realm (RealmName)
      ON DELETE CASCADE
      ON UPDATE CASCADE
) WITHOUT OIDS;


--
--
-- NAME        : Restriction
-- REFERENCES  : Realm
--
-- DESCRIPTION : Ability to block out certain material as the host see's fit
--               including images, and words.

CREATE TABLE ethereal.Restriction (
  RealmName            VARCHAR(30)   NOT NULL DEFAULT 'system',
  RestrictionType      VARCHAR(8)    NOT NULL DEFAULT 'picture',
  RestrictionValue     VARCHAR(255)  NOT NULL,
  CONSTRAINT pkRestriction           PRIMARY KEY (RealmName, RestrictionType, RestrictionValue),
  CONSTRAINT fkRestriction           FOREIGN KEY (RealmName)
    REFERENCES ethereal.Realm (RealmName)
      ON DELETE CASCADE
      ON UPDATE CASCADE,
  CONSTRAINT enumRestrictionType     CHECK (RestrictionType IN ('picture','word'))
) WITHOUT OIDS;


--
--
-- NAME        : Statistics
-- REFERENCES  : Realm
--
-- DESCRIPTION : Is used to gather and be used in the generation of statistics
--               used for reporting.

CREATE TABLE ethereal.Statistics (
  RealmName            VARCHAR(30)   NOT NULL DEFAULT 'system',
  StatTimestamp        TIMESTAMP     NOT NULL DEFAULT LOCALTIMESTAMP,
  StatHits             INT           NOT NULL DEFAULT '0',
  StatPosts            INT           NOT NULL DEFAULT '0',
  CONSTRAINT pkStatistics            PRIMARY KEY (RealmName, StatTimestamp),
  CONSTRAINT fkRealmLocale           FOREIGN KEY (RealmName)
    REFERENCES ethereal.Realm (RealmName)
      ON DELETE CASCADE
      ON UPDATE CASCADE
) WITHOUT OIDS;



--
--
-- NAME        : Puppeteer Restriction
-- REFERENCES  : Realm
--               Puppeteer
--
-- DESCRIPTION : Determines who is barred from accessing the site, or alternatly
--               who can access said realm based on access settings.

CREATE TABLE ethereal.PuppeteerRestriction (
  PuppeteerLogin       VARCHAR(10)   NOT NULL,
  RealmName            VARCHAR(30)   NOT NULL,
  RestrictionTimestamp TIMESTAMP     NOT NULL DEFAULT LOCALTIMESTAMP,
  CONSTRAINT pkPuppeteerRestr             PRIMARY KEY (PuppeteerLogin, RealmName),
  CONSTRAINT fkPuppeteerRestr_Realm       FOREIGN KEY (RealmName)
    REFERENCES ethereal.Realm (RealmName)
      ON DELETE CASCADE
      ON UPDATE CASCADE,
  CONSTRAINT fkPuppeteerRestr_Puppeteer   FOREIGN KEY (PuppeteerLogin)
    REFERENCES ethereal.Puppeteer (PuppeteerLogin)
      ON DELETE CASCADE
      ON UPDATE CASCADE
) WITHOUT OIDS;


--
--
-- NAME        : Realm Supervisor
-- REFERENCES  : Realm
--               Puppeteer
--
-- DESCRIPTION : Realm supervisor dictates who has control over a realm and how
--               much is granted.

CREATE TABLE ethereal.RealmSupervisor (
  RealmName            VARCHAR(30)   NOT NULL,
  PuppeteerLogin       VARCHAR(10)   NOT NULL,
  RealmACL             SMALLINT      NOT NULL DEFAULT '16',
  CONSTRAINT pkRealmSupervisor           PRIMARY KEY (RealmName, PuppeteerLogin),
  CONSTRAINT fkRealmSupervisor_Realm     FOREIGN KEY (RealmName)
    REFERENCES ethereal.Realm (RealmName)
      ON DELETE CASCADE
      ON UPDATE CASCADE,
  CONSTRAINT fkRealmSupervisor_Puppeteer FOREIGN KEY (PuppeteerLogin)
    REFERENCES ethereal.Puppeteer (PuppeteerLogin)
      ON DELETE CASCADE
      ON UPDATE CASCADE
) WITHOUT OIDS;




-- ================================================================================== --
-- PUPPET                                                                             --
-- ================================================================================== --

--
--
-- NAME        : Puppet
-- REFERENCES  : Realm
--               Puppeteer
--
-- DESCRIPTION : Puppet are the entities that are seen to make the actual posts and create
--               activity, even if they are associated directly with Puppeteers.

CREATE TABLE ethereal.Puppet (
  PuppetName           VARCHAR(30)   NOT NULL,
  PuppetPool           VARCHAR(30)   NOT NULL DEFAULT 'system',
  PuppeteerLogin       varchar(10),
  RealmLocale          VARCHAR(25),
  PuppetActive         TIMESTAMP,
  RealmName            VARCHAR(30),
  PuppetAssoication    VARCHAR(10),
  PuppetDefault        VARCHAR(3)    NOT NULL DEFAULT 'no',
  PuppetDisplay        VARCHAR(3)    NOT NULL DEFAULT 'yes',
  PuppetBio            TEXT,
  PuppetTag            TEXT,
  PuppetTitle          TEXT,
  PuppetGummy          VARCHAR(200),
  PuppetLocale         VARCHAR(50),
  PuppetPic            VARCHAR(200),
  PuppetPicHeight      SMALLINT,
  PuppetPicWidth       SMALLINT,
  PuppetPicSize        SMALLINT,
  PuppetTextSize       SMALLINT      NOT NULL DEFAULT '1',
  PuppetTextColour     VARCHAR(25)   NOT NULL DEFAULT 'White',
  PuppetTextFace       VARCHAR(15)   NOT NULL DEFAULT 'Times New Roman',
  CONSTRAINT pkPuppet                PRIMARY KEY (PuppetName, PuppetPool),
  CONSTRAINT fkPuppetRealm           FOREIGN KEY (PuppetPool)
    REFERENCES ethereal.Realm (RealmName)
      ON DELETE CASCADE
      ON UPDATE CASCADE,
  CONSTRAINT fkPuppetPuppeteer       FOREIGN KEY (PuppeteerLogin)
    REFERENCES ethereal.Puppeteer (PuppeteerLogin)
      ON DELETE CASCADE
      ON UPDATE CASCADE,
  CONSTRAINT enumPuppetDefault       CHECK (PuppetDefault IN ('yes','no')),
  CONSTRAINT enumPuppetDisplay       CHECK (PuppetDisplay IN ('yes','no'))
) WITHOUT OIDS;

-- FUNCTION INDEX
CREATE INDEX fncPuppet ON ethereal.Puppet
(
  lower(PuppetName)
);

-- STANDARD INDEX
CREATE INDEX idxPuppet_PuppetName ON ethereal.Puppet
(
  PuppetName
);

CREATE INDEX idxPuppet_Active ON ethereal.Puppet
(
  PuppetActive
);

-- STANDARD INDEX
CREATE INDEX idxPuppet_Default ON ethereal.Puppet
(
  PuppetDefault
);

-- STANDARD INDEX
CREATE INDEX idxPuppet_Association ON ethereal.Puppet
(
  PuppetAssociation
);

--
--
-- NAME        : Board
-- REFERENCES  : Realm
--               BoardTopic
--
-- DESCRIPTION : All posts made to the board, system or otherwise will pass
--               through here.

CREATE TABLE ethereal.Board (
  BoardIDNumber        INT           NOT NULL DEFAULT NEXTVAL('ethereal.seqBoard'),
  RealmName            VARCHAR(30),
  BoardTopic           VARCHAR(25),
  PuppetName           VARCHAR(30),
  BoardSubject         VARCHAR(255),
  BoardTimestamp       TIMESTAMP     NOT NULL DEFAULT LOCALTIMESTAMP,
  BoardUpdate          TIMESTAMP     NOT NULL DEFAULT LOCALTIMESTAMP,
  BoardMessage         TEXT,
  BoardLevel           SMALLINT      NOT NULL DEFAULT '0',
  LevelZero            INT           NOT NULL DEFAULT '0',
  LevelOne             INT           NOT NULL DEFAULT '0',
  LevelTwo             INT           NOT NULL DEFAULT '0',
  LevelThree           INT           NOT NULL DEFAULT '0',
  LevelFour            INT           NOT NULL DEFAULT '0',
  LevelFive            INT           NOT NULL DEFAULT '0',
  CONSTRAINT pkBoard                 PRIMARY KEY (BoardIDNumber),
  CONSTRAINT fkBoard                 FOREIGN KEY (RealmName, BoardTopic)
    REFERENCES ethereal.BoardTopic (RealmName, BoardTopic)
      ON DELETE CASCADE
      ON UPDATE CASCADE
) WITHOUT OIDS;

-- STANDARD INDEX
CREATE INDEX idxBoard_Topic ON ethereal.Board
(
  BoardTopic
);

CREATE INDEX idxBoard_Timestamp ON ethereal.Board
(
  BoardTimestamp
);

CREATE INDEX idxBoard_Update ON ethereal.Board
(
  BoardUpdate
);

CREATE INDEX idxBoard_Zero ON ethereal.Board
(
  LevelZero
);

CREATE INDEX idxBoard_One ON ethereal.Board
(
  LevelOne
);

CREATE INDEX idxBoard_Two ON ethereal.Board
(
  LevelTwo
);

CREATE INDEX idxBoard_Three ON ethereal.Board
(
  LevelThree
);

CREATE INDEX idxBoard_Four ON ethereal.Board
(
  LevelFour
);

CREATE INDEX idxBoard_Five ON ethereal.Board
(
  LevelFive
);



--
--
-- NAME        : Mail
-- REFERENCES  : Puppeteer
--
-- DESCRIPTION : Mail allows us to make use of a small internal messaging system which
--               can be used to contact people even offline.

CREATE TABLE ethereal.Mail (
  MailIDNumber         INT           NOT NULL DEFAULT NEXTVAL('ethereal.seqMail'),
  PuppeteerLogin       VARCHAR(10)   NOT NULL,
  MailFrom             VARCHAR(30)   NOT NULL,
  MailRcpt             VARCHAR(30)   NOT NULL,
  MailSubject          VARCHAR(250),
  MailTimestamp        TIMESTAMP     NOT NULL DEFAULT LOCALTIMESTAMP,
  MailContents         TEXT,
  CONSTRAINT pkMail                  PRIMARY KEY (MailIDNumber)
) WITHOUT OIDS;

-- STANDARD INDEX
CREATE INDEX idxMail_Login ON ethereal.Mail
(
       PuppeteerLogin
);

CREATE INDEX idxMail_Timestamp ON ethereal.Mail
(
       MailTimestamp
);

--
--
-- NAME        : Puppet Ignore
-- REFERENCES  : Puppeteer
--               Puppet
--
-- DESCRIPTION : Allows people to ignore based on specific characteristics while
--               in a realm.

CREATE TABLE ethereal.PuppetIgnore (
  PuppeteerLogin       VARCHAR(10)   NOT NULL,
  PuppetName           VARCHAR(30)   NOT NULL,
  PuppetLogin          VARCHAR(10),
  PuppetIgnore         VARCHAR(6)    NOT NULL,
  CONSTRAINT pkPuppetIgnore          PRIMARY KEY (PuppeteerLogin, PuppetName),
  CONSTRAINT fkPuppetIgnorePuppeteer FOREIGN KEY (PuppeteerLogin)
    REFERENCES ethereal.Puppeteer (PuppeteerLogin)
      ON DELETE CASCADE
      ON UPDATE CASCADE,
  CONSTRAINT enumPuppetIgnore        CHECK (PuppetIgnore IN ('behead','single','global','visual','optics'))
) WITHOUT OIDS;

-- STANDARD INDEX
CREATE INDEX idxIgnore_Ignore ON ethereal.PuppetIgnore
(
       PuppetIgnore
);

-- MULTI COLUMN
CREATE INDEX idxIgnore_LoginIgnore ON ethereal.PuppetIgnore
(
       Puppeteerlogin,
       PuppetIgnore
);

--
--
-- NAME        : Weblog
-- REFERENCES  : NIL
--
-- DESCRIPTION : As the name implies, this table stores weblog entries made for the
--               system.  Allowing people to be informed of new changes et cetera.

CREATE TABLE ethereal.Weblog (
  LogIDNumber          INT           NOT NULL DEFAULT NEXTVAL('ethereal.seqWeblog'),
  PuppetName           VARCHAR(30)   NOT NULL,
  LogTitle             VARCHAR(100)  NOT NULL,
  LogTopic             VARCHAR(25)   NOT NULL,
  LogTimestamp         TIMESTAMP     NOT NULL DEFAULT LOCALTIMESTAMP,
  LogPost              TEXT          NOT NULL,
  CONSTRAINT pkWeblog                PRIMARY KEY (LogIDNumber)

) WITHOUT OIDS;


-- STANDARD INDEX
CREATE INDEX idxWeblog_Title ON ethereal.Weblog
(
  LogTitle
);

CREATE INDEX idxWeblog_Timestamp ON ethereal.Weblog
(
  LogTimestamp
);

CREATE INDEX idxWeblog_Topic ON ethereal.Weblog
(
  LogTopic
);


-- ================================================================================== --
-- POST                                                                               --
-- ================================================================================== --

--
--
-- NAME        : Post
-- REFERENCES  : Realm*
--               Puppet*
--               PuppeteerLogin*
--
-- DESCRIPTION : Post is the hive of activity for all realms.  Associated with all three
--               major tables, it is not actually linked because of the nature of storing
--               posts for statistics and auditing.

CREATE TABLE ethereal.Post (
  PostIDNumber         INT           NOT NULL DEFAULT NEXTVAL('ethereal.seqPost'),
  RealmName            VARCHAR(30)   NOT NULL,
  PuppetName           VARCHAR(30),
  PuppeteerLogin       VARCHAR(10),
  PostTo               VARCHAR(30),
  PostTimestamp        TIMESTAMP     NOT NULL DEFAULT LOCALTIMESTAMP,
  PostClass            VARCHAR(10)   NOT NULL DEFAULT 'general',
  PostCmd              VARCHAR(10)   NOT NULL DEFAULT 'none',
  PostFullFormat       TEXT,
  PostImagelessFormat  TEXT,
  PostPartialFormat    TEXT,
  CONSTRAINT pkPost                  PRIMARY KEY (PostIDNumber),
  CONSTRAINT enumPostClass           CHECK (PostCLass IN ('banner','dice','duplicate','general','play','private','special','system')),
  CONSTRAINT enumPostCmd             CHECK (PostCmd IN ('general','none','play','stream'))
) WITHOUT OIDS;

-- STANDARD INDEX
CREATE INDEX idxPost_Class ON ethereal.Post
(
  PostClass
);

CREATE INDEX idxPost_Login ON ethereal.Post
(
  PuppeteerLogin
);

CREATE INDEX idxPost_Puppet ON ethereal.Post
(
  PuppetName
);

CREATE INDEX idxPost_Realm ON ethereal.Post
(
  RealmName
);

CREATE INDEX idxPost_Timestamp ON ethereal.Post
(
  PostTimestamp
);

-- MULTI COLUMN
CREATE INDEX idxPost_RealmClassTo ON ethereal.Post
(
  RealmName,
  PostClass,
  PostTo
);

CREATE INDEX idxPost_NumberRealm ON ethereal.Post
(
  PostIDNumber,
  RealmName
);

CREATE INDEX idxPost_TimestampPuppet ON ethereal.Post
(
  PostTimestamp,
  PuppetName
);

-- ================================================================================== --
-- VIEWS                                                                              --
-- ================================================================================== --

--
--
-- NAME        : Exhibition
--
-- DESCRIPTION : This combines the gallery and work tables in order to return information
--               that is crucial for listing galleries.  

CREATE VIEW ethereal.Exhibition AS
   SELECT
     gal.GalleryID          AS GalleryID,
     gal.PuppeteerLogin     AS PuppeteerLogin,
     gal.GalleryPenName     AS GalleryPenName,
     gal.GalleryName        AS GalleryName,
     gal.GalleryRating      AS GalleryRating,
     gal.GalleryDescription AS GalleryDescription,
     gal.GalleryType        AS GalleryType,
     gre.GenreName          AS GalleryGenre,
     gal.GalleryCreated     AS GalleryCreated,
     gal.GalleryUpdated     AS GalleryUpdated,
     (COUNT(wrk.WorkID)-2)  AS WorkCount
   FROM ethereal.Gallery gal, ethereal.Work wrk, ethereal.GalleryGenre gre
   WHERE gal.GalleryID = wrk.GalleryID
     AND gal.GalleryID = gre.GalleryID
   GROUP BY gal.GalleryID,
            gal.PuppeteerLogin,
            gal.GalleryPenName,
            gal.GalleryName,
            gal.GalleryRating,
            gal.GalleryDescription,
            gal.GalleryType,
            gal.GalleryCreated,
            gal.GalleryUpdated,
            gre.GenreName
   HAVING COUNT(wrk.WorkID) > 2::INT;

--
--
-- NAME        : Portal
--
-- DESCRIPTION : This is used exclusively with the portal, as to allow it to get information
--               already in a form that it can use and send people off to the next screen.

CREATE VIEW ethereal.Portal AS
   SELECT
     Po.PuppetName AS PuppetName,
     Po.RealmName  AS RealmName
   FROM ethereal.Post Po, ethereal.Realm Re
   WHERE Po.RealmName = Re.RealmName
     AND (Re.RealmPublic='yes'
      OR Re.RealmVerified='yes')
    AND Po.PostTimestamp > ethereal.GetInterval('10 Minutes')
    AND Po.PuppetName IS NOT NULL
  GROUP BY Po.RealmName, Po.PuppetName;

--
--
-- NAME        : Warning
--
-- DESCRIPTION : This will allow an internal script to see in one sweep how many occurences
--               took place within three days specifically.    Simplifying the overall
--               function used.

CREATE VIEW ethereal.Warning AS
   SELECT
      AuditRealm            AS RealmName,
      AuditAdditional       AS PuppeteerLogin,
      DATE_TRUNC('day', AuditTimestamp) AS WarnTimestamp
   FROM ethereal.Audit
   WHERE AuditCode=2007::INT
     AND AuditTimestamp > (LOCALTIMESTAMP - INTERVAL '3 days');


--
--
-- NAME        : Who is On
--
-- DESCRIPTION : This is a pre-queries list of who is on.  Which allows for additional
--               fine tuning generally required to get much of anything done.

CREATE VIEW ethereal.Who AS
 SELECT
    Po.PuppetName     AS PuppetName,
    Po.PuppeteerLogin AS PuppeteerLogin,
    Po.RealmName      AS RealmName,
    Re.RealmPublic    AS RealmPublic,
    Re.RealmVerified  AS RealmVerified
  FROM ethereal.Post Po, ethereal.Puppet Ch, ethereal.Realm Re
  WHERE Po.PuppeteerLogin = Ch.PuppeteerLogin
    AND Po.RealmName = Re.RealmName
    AND Po.PostTimestamp > ethereal.getInterval('10 minutes')
    AND Po.PuppetName IS NOT NULL
  GROUP BY Po.PuppeteerLogin, Po.PuppetName, Po.RealmName, Re.RealmPublic, Re.RealmVerified;





-- ================================================================================== --
-- OWNERSHIP                                                                          --
-- ================================================================================== --

-- TABLES
ALTER TABLE ethereal.Association           OWNER TO ethereal;
ALTER TABLE ethereal.Audit                 OWNER TO ethereal;
ALTER TABLE ethereal.Board                 OWNER TO ethereal;
ALTER TABLE ethereal.BoardTopic            OWNER TO ethereal;
ALTER TABLE ethereal.Comment               OWNER TO ethereal;
ALTER TABLE ethereal.Contact               OWNER TO ethereal;
ALTER TABLE ethereal.Font                  OWNER TO ethereal;
ALTER TABLE ethereal.Fortune               OWNER TO ethereal;
ALTER TABLE ethereal.Gallery               OWNER TO ethereal;
ALTER TABLE ethereal.GalleryGenre          OWNER TO ethereal;
ALTER TABLE ethereal.GalleryNotify         OWNER TO ethereal;
ALTER TABLE ethereal.Genre                 OWNER TO ethereal;
ALTER TABLE ethereal.Gummy                 OWNER TO ethereal;
ALTER TABLE ethereal.Mail                  OWNER TO ethereal;
ALTER TABLE ethereal.Param                 OWNER TO ethereal;
ALTER TABLE ethereal.Poll                  OWNER TO ethereal;
ALTER TABLE ethereal.PollOption            OWNER TO ethereal;
ALTER TABLE ethereal.PollVote              OWNER TO ethereal;
ALTER TABLE ethereal.Post                  OWNER TO ethereal;
ALTER TABLE ethereal.Puppet                OWNER TO ethereal;
ALTER TABLE ethereal.PuppetIgnore          OWNER TO ethereal;
ALTER TABLE ethereal.Puppeteer             OWNER TO ethereal;
ALTER TABLE ethereal.PuppeteerContact      OWNER TO ethereal;
ALTER TABLE ethereal.PuppeteerRegistration OWNER TO ethereal;
ALTER TABLE ethereal.PuppeteerRestriction  OWNER TO ethereal;
ALTER TABLE ethereal.Realm                 OWNER TO ethereal;
ALTER TABLE ethereal.RealmDesign           OWNER TO ethereal;
ALTER TABLE ethereal.RealmLocale           OWNER TO ethereal;
ALTER TABLE ethereal.RealmSupervisor       OWNER TO ethereal;
ALTER TABLE ethereal.Restriction           OWNER TO ethereal;
ALTER TABLE ethereal.Rotate                OWNER TO ethereal;
ALTER TABLE ethereal.Statistics            OWNER TO ethereal;
ALTER TABLE ethereal.Tag                   OWNER TO ethereal;
ALTER TABLE ethereal.Transaction           OWNER TO ethereal;
ALTER TABLE ethereal.Weblog                OWNER TO ethereal;
ALTER TABLE ethereal.Work                  OWNER TO ethereal;

-- SEQUENCES
ALTER TABLE ethereal.seqAudit              OWNER TO ethereal;
ALTER TABLE ethereal.seqBoard              OWNER TO ethereal;
ALTER TABLE ethereal.seqComment            OWNER TO ethereal;
ALTER TABLE ethereal.seqFortune            OWNER TO ethereal;
ALTER TABLE ethereal.seqGallery            OWNER TO ethereal;
ALTER TABLE ethereal.seqMail               OWNER TO ethereal;
ALTER TABLE ethereal.seqOption             OWNER TO ethereal;
ALTER TABLE ethereal.seqPoll               OWNER TO ethereal;
ALTER TABLE ethereal.seqPost               OWNER TO ethereal;
ALTER TABLE ethereal.seqWeblog             OWNER TO ethereal;
ALTER TABLE ethereal.seqWork               OWNER TO ethereal;

-- VIEWS
ALTER TABLE ethereal.Exhibition            OWNER TO ethereal;
ALTER TABLE ethereal.Portal                OWNER TO ethereal;
ALTER TABLE ethereal.Warning               OWNER TO ethereal;
ALTER TABLE ethereal.Who                   OWNER TO ethereal;

-- NOTES:
--
-- * Will not actually reference due to nature of posts being stored indefinitly.
--
